@@ -28,8 +28,16 @@ The corresponding server must be started before e.g. as:
 """
 import asyncio
 import logging
+import sys
 
-import helper
+
+try:
+    import helper
+except ImportError:
+    print("*** ERROR --> THIS EXAMPLE needs the example directory, please see \n\
+          https://pymodbus.readthedocs.io/en/latest/source/examples.html\n\
+          for more information.")
+    sys.exit(-1)
 
 import pymodbus.client as modbusClient
 from pymodbus import ModbusException
@@ -45,6 +53,7 @@ def setup_async_client(description=None, cmdline=None):
         server=False, description=description, cmdline=cmdline
     )
     _logger.info("### Create client object")
+    client = None
     if args.comm == "tcp":
         client = modbusClient.AsyncModbusTcpClient(
             args.host,
@@ -85,6 +94,7 @@ def setup_async_client(description=None, cmdline=None):
             #    parity="N",
             #    stopbits=1,
             #    handle_local_echo=False,
+            #    strict=True,
         )
     elif args.comm == "tls":
         client = modbusClient.AsyncModbusTlsClient(
@@ -95,8 +105,6 @@ def setup_async_client(description=None, cmdline=None):
             timeout=args.timeout,
             #    retries=3,
             #    retry_on_empty=False,
-            #    close_comm_on_error=False,
-            #    strict=True,
             # TLS setup parameters
             #    sslctx=sslctx,
             certfile=helper.get_certificate("crt"),
