@@ -1,13 +1,18 @@
 """Bit Reading Request/Response messages."""
+
+__all__ = [
+    "ReadBitsResponseBase",
+    "ReadCoilsRequest",
+    "ReadCoilsResponse",
+    "ReadDiscreteInputsRequest",
+    "ReadDiscreteInputsResponse",
+]
+
 # pylint: disable=missing-type-doc
 import struct
 
-from pymodbus.constants import Defaults
-from pymodbus.pdu import (
-    ModbusExceptions as merror,
-    ModbusRequest,
-    ModbusResponse,
-)
+from pymodbus.pdu import ModbusExceptions as merror
+from pymodbus.pdu import ModbusRequest, ModbusResponse
 from pymodbus.utilities import pack_bitstring, unpack_bitstring
 
 
@@ -16,14 +21,14 @@ class ReadBitsRequestBase(ModbusRequest):
 
     _rtu_frame_size = 8
 
-    def __init__(self, address, count, unit=Defaults.Slave, **kwargs):
+    def __init__(self, address, count, slave=0, **kwargs):
         """Initialize the read request data.
 
         :param address: The start address to read from
         :param count: The number of bits after "address" to read
-        :param unit: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         """
-        ModbusRequest.__init__(self, unit, **kwargs)
+        ModbusRequest.__init__(self, slave, **kwargs)
         self.address = address
         self.count = count
 
@@ -70,13 +75,13 @@ class ReadBitsResponseBase(ModbusResponse):
 
     _rtu_byte_count_pos = 2
 
-    def __init__(self, values, unit=Defaults.Slave, **kwargs):
+    def __init__(self, values, slave=0, **kwargs):
         """Initialize a new instance.
 
         :param values: The requested values to be returned
-        :param unit: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         """
-        ModbusResponse.__init__(self, unit, **kwargs)
+        ModbusResponse.__init__(self, slave, **kwargs)
 
         #: A list of booleans representing bit values
         self.bits = values or []
@@ -98,7 +103,7 @@ class ReadBitsResponseBase(ModbusResponse):
         self.byte_count = int(data[0])  # pylint: disable=attribute-defined-outside-init
         self.bits = unpack_bitstring(data[1:])
 
-    def setBit(self, address, value=1):  # pylint: disable=invalid-name
+    def setBit(self, address, value=1):
         """Set the specified bit.
 
         :param address: The bit to set
@@ -106,15 +111,15 @@ class ReadBitsResponseBase(ModbusResponse):
         """
         self.bits[address] = bool(value)
 
-    def resetBit(self, address):  # pylint: disable=invalid-name
+    def resetBit(self, address):
         """Set the specified bit to 0.
 
         :param address: The bit to reset
         """
         self.setBit(address, 0)
 
-    def getBit(self, address):  # pylint: disable=invalid-name
-        """Get the specified bit"s value.
+    def getBit(self, address):
+        """Get the specified bit's value.
 
         :param address: The bit to query
         :returns: The value of the requested bit
@@ -139,17 +144,18 @@ class ReadCoilsRequest(ReadBitsRequestBase):
     """
 
     function_code = 1
+    function_code_name = "read_coils"
 
-    def __init__(self, address=None, count=None, unit=Defaults.Slave, **kwargs):
+    def __init__(self, address=None, count=None, slave=0, **kwargs):
         """Initialize a new instance.
 
         :param address: The address to start reading from
         :param count: The number of bits to read
-        :param unit: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         """
-        ReadBitsRequestBase.__init__(self, address, count, unit, **kwargs)
+        ReadBitsRequestBase.__init__(self, address, count, slave, **kwargs)
 
-    def execute(self, context):
+    async def execute(self, context):
         """Run a read coils request against a datastore.
 
         Before running the request, we make sure that the request is in
@@ -163,7 +169,9 @@ class ReadCoilsRequest(ReadBitsRequestBase):
             return self.doException(merror.IllegalValue)
         if not context.validate(self.function_code, self.address, self.count):
             return self.doException(merror.IllegalAddress)
-        values = context.getValues(self.function_code, self.address, self.count)
+        values = await context.async_getValues(
+            self.function_code, self.address, self.count
+        )
         return ReadCoilsResponse(values)
 
 
@@ -185,13 +193,13 @@ class ReadCoilsResponse(ReadBitsResponseBase):
 
     function_code = 1
 
-    def __init__(self, values=None, unit=Defaults.Slave, **kwargs):
+    def __init__(self, values=None, slave=0, **kwargs):
         """Initialize a new instance.
 
         :param values: The request values to respond with
-        :param unit: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         """
-        ReadBitsResponseBase.__init__(self, values, unit, **kwargs)
+        ReadBitsResponseBase.__init__(self, values, slave, **kwargs)
 
 
 class ReadDiscreteInputsRequest(ReadBitsRequestBase):
@@ -204,17 +212,18 @@ class ReadDiscreteInputsRequest(ReadBitsRequestBase):
     """
 
     function_code = 2
+    function_code_name = "read_discrete_input"
 
-    def __init__(self, address=None, count=None, unit=Defaults.Slave, **kwargs):
+    def __init__(self, address=None, count=None, slave=0, **kwargs):
         """Initialize a new instance.
 
         :param address: The address to start reading from
         :param count: The number of bits to read
-        :param unit: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         """
-        ReadBitsRequestBase.__init__(self, address, count, unit, **kwargs)
+        ReadBitsRequestBase.__init__(self, address, count, slave, **kwargs)
 
-    def execute(self, context):
+    async def execute(self, context):
         """Run a read discrete input request against a datastore.
 
         Before running the request, we make sure that the request is in
@@ -228,7 +237,9 @@ class ReadDiscreteInputsRequest(ReadBitsRequestBase):
             return self.doException(merror.IllegalValue)
         if not context.validate(self.function_code, self.address, self.count):
             return self.doException(merror.IllegalAddress)
-        values = context.getValues(self.function_code, self.address, self.count)
+        values = await context.async_getValues(
+            self.function_code, self.address, self.count
+        )
         return ReadDiscreteInputsResponse(values)
 
 
@@ -250,22 +261,10 @@ class ReadDiscreteInputsResponse(ReadBitsResponseBase):
 
     function_code = 2
 
-    def __init__(self, values=None, unit=Defaults.Slave, **kwargs):
+    def __init__(self, values=None, slave=0, **kwargs):
         """Initialize a new instance.
 
         :param values: The request values to respond with
-        :param unit: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         """
-        ReadBitsResponseBase.__init__(self, values, unit, **kwargs)
-
-
-# ---------------------------------------------------------------------------#
-#  Exported symbols
-# ---------------------------------------------------------------------------#
-__all__ = [
-    "ReadBitsResponseBase",
-    "ReadCoilsRequest",
-    "ReadCoilsResponse",
-    "ReadDiscreteInputsRequest",
-    "ReadDiscreteInputsResponse",
-]
+        ReadBitsResponseBase.__init__(self, values, slave, **kwargs)
