@@ -14,8 +14,8 @@ __all__ = [
 import struct
 
 from pymodbus.constants import ModbusStatus
-from pymodbus.pdu import ExceptionResponse, ModbusRequest, ModbusResponse
 from pymodbus.pdu import ModbusExceptions as merror
+from pymodbus.pdu import ModbusRequest, ModbusResponse
 from pymodbus.utilities import pack_bitstring, unpack_bitstring
 
 
@@ -79,7 +79,7 @@ class WriteSingleCoilRequest(ModbusRequest):
         self.address, value = struct.unpack(">HH", data)
         self.value = value == ModbusStatus.ON
 
-    def execute(self, context):
+    async def execute(self, context):
         """Run a write coil request against a datastore.
 
         :param context: The datastore to request from
@@ -90,12 +90,8 @@ class WriteSingleCoilRequest(ModbusRequest):
         if not context.validate(self.function_code, self.address, 1):
             return self.doException(merror.IllegalAddress)
 
-        result = context.setValues(self.function_code, self.address, [self.value])
-        if isinstance(result, ExceptionResponse):
-            return result
-        values = context.getValues(self.function_code, self.address, 1)
-        if isinstance(values, ExceptionResponse):
-            return values
+        await context.async_setValues(self.function_code, self.address, [self.value])
+        values = await context.async_getValues(self.function_code, self.address, 1)
         return WriteSingleCoilResponse(self.address, values[0])
 
     def get_response_pdu_size(self):
@@ -212,7 +208,7 @@ class WriteMultipleCoilsRequest(ModbusRequest):
         values = unpack_bitstring(data[5:])
         self.values = values[:count]
 
-    def execute(self, context):
+    async def execute(self, context):
         """Run a write coils request against a datastore.
 
         :param context: The datastore to request from
@@ -226,9 +222,9 @@ class WriteMultipleCoilsRequest(ModbusRequest):
         if not context.validate(self.function_code, self.address, count):
             return self.doException(merror.IllegalAddress)
 
-        result = context.setValues(self.function_code, self.address, self.values)
-        if isinstance(result, ExceptionResponse):
-            return result
+        await context.async_setValues(
+            self.function_code, self.address, self.values
+        )
         return WriteMultipleCoilsResponse(self.address, count)
 
     def __str__(self):
