@@ -4,7 +4,7 @@ from __future__ import annotations
 import asyncio
 import socket
 from dataclasses import dataclass
-from typing import Any, Callable, Type, cast
+from typing import Any, Awaitable, Callable, Type, cast
 
 from pymodbus.client.mixin import ModbusClientMixin
 from pymodbus.exceptions import ConnectionException, ModbusIOException
@@ -12,12 +12,12 @@ from pymodbus.factory import ClientDecoder
 from pymodbus.framer import FRAMER_NAME_TO_CLASS, Framer, ModbusFramer
 from pymodbus.logging import Log
 from pymodbus.pdu import ModbusRequest, ModbusResponse
-from pymodbus.transaction import DictTransactionManager
+from pymodbus.transaction import ModbusTransactionManager
 from pymodbus.transport import CommParams, ModbusProtocol
 from pymodbus.utilities import ModbusTransactionState
 
 
-class ModbusBaseClient(ModbusClientMixin, ModbusProtocol):
+class ModbusBaseClient(ModbusClientMixin[Awaitable[ModbusResponse]], ModbusProtocol):
     """**ModbusBaseClient**.
 
     Fixed parameters:
@@ -60,7 +60,7 @@ class ModbusBaseClient(ModbusClientMixin, ModbusProtocol):
         **kwargs: Any,
     ) -> None:
         """Initialize a client instance."""
-        ModbusClientMixin.__init__(self)
+        ModbusClientMixin.__init__(self)  # type: ignore[arg-type]
         ModbusProtocol.__init__(
             self,
             CommParams(
@@ -92,13 +92,14 @@ class ModbusBaseClient(ModbusClientMixin, ModbusProtocol):
         self.framer = FRAMER_NAME_TO_CLASS.get(
             framer, cast(Type[ModbusFramer], framer)
         )(ClientDecoder(), self)
-        self.transaction = DictTransactionManager(
+        self.transaction = ModbusTransactionManager(
             self, retries=retries, retry_on_empty=retry_on_empty, **kwargs
         )
         self.use_udp = False
         self.state = ModbusTransactionState.IDLE
         self.last_frame_end: float | None = 0
         self.silent_interval: float = 0
+        self._lock = asyncio.Lock()
 
     # ----------------------------------------------------------------------- #
     # Client external interface
@@ -108,6 +109,11 @@ class ModbusBaseClient(ModbusClientMixin, ModbusProtocol):
         """Return state of connection."""
         return self.is_active()
 
+    async def base_connect(self) -> bool:
+        """Call transport connect."""
+        return await super().connect()
+
+
     def register(self, custom_response_class: ModbusResponse) -> None:
         """Register a custom response class with the decoder (call **sync**).
 
@@ -124,7 +130,7 @@ class ModbusBaseClient(ModbusClientMixin, ModbusProtocol):
         if reconnect:
             self.connection_lost(asyncio.TimeoutError("Server not responding"))
         else:
-            self.transport_close()
+            super().close()
 
     def idle_time(self) -> float:
         """Time before initiating next transaction (call **sync**).
@@ -136,7 +142,7 @@ class ModbusBaseClient(ModbusClientMixin, ModbusProtocol):
             return 0
         return self.last_frame_end + self.silent_interval
 
-    def execute(self, request: ModbusRequest | None = None) -> ModbusResponse:
+    def execute(self, request: ModbusRequest | None = None):
         """Execute request and get response (call **sync/async**).
 
         :param request: The request to process
@@ -150,33 +156,48 @@ class ModbusBaseClient(ModbusClientMixin, ModbusProtocol):
     # ----------------------------------------------------------------------- #
     # Merged client methods
     # ----------------------------------------------------------------------- #
-    async def async_execute(self, request=None):
+    async def async_execute(self, request) -> ModbusResponse:
         """Execute requests asynchronously."""
         request.transaction_id = self.transaction.getNextTID()
         packet = self.framer.buildPacket(request)
 
         count = 0
         while count <= self.retries:
-            req = self.build_response(request.transaction_id)
-            if not count or not self.no_resend_on_retry:
-                self.transport_send(packet)
-            if self.broadcast_enable and not request.slave_id:
-                resp = b"Broadcast write sent - no response expected"
-                break
-            try:
-                resp = await asyncio.wait_for(
-                    req, timeout=self.comm_params.timeout_connect
-                )
-                break
-            except asyncio.exceptions.TimeoutError:
-                count += 1
+            async with self._lock:
+                req = self.build_response(request.transaction_id)
+                if not count or not self.no_resend_on_retry:
+                    self.framer.resetFrame()
+                    self.send(packet)
+                if self.broadcast_enable and not request.slave_id:
+                    resp = None
+                    break
+                try:
+                    resp = await asyncio.wait_for(
+                        req, timeout=self.comm_params.timeout_connect
+                    )
+                    break
+                except asyncio.exceptions.TimeoutError:
+                    count += 1
         if count > self.retries:
             self.close(reconnect=True)
             raise ModbusIOException(
                 f"ERROR: No response received after {self.retries} retries"
             )
 
-        return resp
+        return resp  # type: ignore[return-value]
+
+    def callback_new_connection(self):
+        """Call when listener receive new connection request."""
+
+    def callback_connected(self) -> None:
+        """Call when connection is succcesfull."""
+        if self.on_reconnect_callback:
+            self.on_reconnect_callback()
+        self.framer.resetFrame()
+
+    def callback_disconnected(self, exc: Exception | None) -> None:
+        """Call when connection is lost."""
+        Log.debug("callback_disconnected called: {}", exc)
 
     def callback_data(self, data: bytes, addr: tuple | None = None) -> int:
         """Handle received data.
@@ -186,7 +207,7 @@ class ModbusBaseClient(ModbusClientMixin, ModbusProtocol):
         self.framer.processIncomingPacket(data, self._handle_response, slave=0)
         return len(data)
 
-    async def connect(self):
+    async def connect(self) -> bool:  # type: ignore[empty-body]
         """Connect to the modbus remote host."""
 
     def raise_future(self, my_future, exc):
@@ -206,7 +227,7 @@ class ModbusBaseClient(ModbusClientMixin, ModbusProtocol):
 
     def build_response(self, tid):
         """Return a deferred response for the current request."""
-        my_future = asyncio.Future()
+        my_future: asyncio.Future = asyncio.Future()
         if not self.transport:
             self.raise_future(my_future, ConnectionException("Client is not connected"))
         else:
@@ -216,12 +237,6 @@ class ModbusBaseClient(ModbusClientMixin, ModbusProtocol):
     # ----------------------------------------------------------------------- #
     # Internal methods
     # ----------------------------------------------------------------------- #
-    def send(self, request):
-        """Send request.
-
-        :meta private:
-        """
-
     def recv(self, size):
         """Receive data.
 
@@ -231,15 +246,6 @@ class ModbusBaseClient(ModbusClientMixin, ModbusProtocol):
     # ----------------------------------------------------------------------- #
     # The magic methods
     # ----------------------------------------------------------------------- #
-    def __enter__(self):
-        """Implement the client with enter block.
-
-        :returns: The current instance of the client
-        :raises ConnectionException:
-        """
-        self.connect()
-        return self
-
     async def __aenter__(self):
         """Implement the client with enter block.
 
@@ -249,12 +255,8 @@ class ModbusBaseClient(ModbusClientMixin, ModbusProtocol):
         await self.connect()
         return self
 
-    def __exit__(self, klass, value, traceback):
-        """Implement the client with exit block."""
-        self.close()
-
     async def __aexit__(self, klass, value, traceback):
-        """Implement the client with exit block."""
+        """Implement the client with aexit block."""
         self.close()
 
     def __str__(self):
@@ -266,7 +268,8 @@ class ModbusBaseClient(ModbusClientMixin, ModbusProtocol):
             f"{self.__class__.__name__} {self.comm_params.host}:{self.comm_params.port}"
         )
 
-class ModbusBaseSyncClient(ModbusClientMixin, ModbusProtocol):
+
+class ModbusBaseSyncClient(ModbusClientMixin[ModbusResponse]):
     """**ModbusBaseClient**.
 
     Fixed parameters:
@@ -278,8 +281,6 @@ class ModbusBaseSyncClient(ModbusClientMixin, ModbusProtocol):
     :param timeout: Timeout for a request, in seconds.
     :param retries: Max number of retries per request.
     :param retry_on_empty: Retry on empty response.
-    :param close_comm_on_error: Close connection on error.
-    :param strict: Strict timing, 1.5 character between requests.
     :param broadcast_enable: True to treat id 0 as broadcast address.
     :param reconnect_delay: Minimum delay in seconds.milliseconds before reconnecting.
     :param reconnect_delay_max: Maximum delay in seconds.milliseconds before reconnecting.
@@ -303,58 +304,43 @@ class ModbusBaseSyncClient(ModbusClientMixin, ModbusProtocol):
 
         retries: int | None = None
         retry_on_empty: bool | None = None
-        close_comm_on_error: bool | None = None
-        strict: bool | None = None
         broadcast_enable: bool | None = None
         reconnect_delay: int | None = None
-
         source_address: tuple[str, int] | None = None
 
-        server_hostname: str | None = None
-
-    def __init__(  # pylint: disable=too-many-arguments
+    def __init__(
         self,
         framer: Framer,
         timeout: float = 3,
         retries: int = 3,
         retry_on_empty: bool = False,
-        close_comm_on_error: bool = False,
-        strict: bool = True,
         broadcast_enable: bool = False,
         reconnect_delay: float = 0.1,
         reconnect_delay_max: float = 300.0,
-        on_reconnect_callback: Callable[[], None] | None = None,
         no_resend_on_retry: bool = False,
         **kwargs: Any,
     ) -> None:
         """Initialize a client instance."""
-        ModbusClientMixin.__init__(self)
-        ModbusProtocol.__init__(
-            self,
-            CommParams(
-                comm_type=kwargs.get("CommType"),
-                comm_name="comm",
-                source_address=kwargs.get("source_address", None),
-                reconnect_delay=reconnect_delay,
-                reconnect_delay_max=reconnect_delay_max,
-                timeout_connect=timeout,
-                host=kwargs.get("host", None),
-                port=kwargs.get("port", 0),
-                sslctx=kwargs.get("sslctx", None),
-                baudrate=kwargs.get("baudrate", None),
-                bytesize=kwargs.get("bytesize", None),
-                parity=kwargs.get("parity", None),
-                stopbits=kwargs.get("stopbits", None),
-                handle_local_echo=kwargs.get("handle_local_echo", False),
-                on_reconnect_callback = on_reconnect_callback,
-            ),
-            False,
+        ModbusClientMixin.__init__(self)  # type: ignore[arg-type]
+        self.comm_params = CommParams(
+            comm_type=kwargs.get("CommType"),
+            comm_name="comm",
+            source_address=kwargs.get("source_address", None),
+            reconnect_delay=reconnect_delay,
+            reconnect_delay_max=reconnect_delay_max,
+            timeout_connect=timeout,
+            host=kwargs.get("host", None),
+            port=kwargs.get("port", 0),
+            sslctx=kwargs.get("sslctx", None),
+            baudrate=kwargs.get("baudrate", None),
+            bytesize=kwargs.get("bytesize", None),
+            parity=kwargs.get("parity", None),
+            stopbits=kwargs.get("stopbits", None),
+            handle_local_echo=kwargs.get("handle_local_echo", False),
         )
         self.params = self._params()
         self.params.retries = int(retries)
         self.params.retry_on_empty = bool(retry_on_empty)
-        self.params.close_comm_on_error = bool(close_comm_on_error)
-        self.params.strict = bool(strict)
         self.params.broadcast_enable = bool(broadcast_enable)
         self.retry_on_empty: int = 0
         self.no_resend_on_retry = no_resend_on_retry
@@ -364,7 +350,7 @@ class ModbusBaseSyncClient(ModbusClientMixin, ModbusProtocol):
         self.framer = FRAMER_NAME_TO_CLASS.get(
             framer, cast(Type[ModbusFramer], framer)
         )(ClientDecoder(), self)
-        self.transaction = DictTransactionManager(
+        self.transaction = ModbusTransactionManager(
             self, retries=retries, retry_on_empty=retry_on_empty, **kwargs
         )
         self.reconnect_delay_current = self.params.reconnect_delay or 0
@@ -372,6 +358,7 @@ class ModbusBaseSyncClient(ModbusClientMixin, ModbusProtocol):
         self.state = ModbusTransactionState.IDLE
         self.last_frame_end: float | None = 0
         self.silent_interval: float = 0
+        self.transport = None
 
     # ----------------------------------------------------------------------- #
     # Client external interface
@@ -437,7 +424,7 @@ class ModbusBaseSyncClient(ModbusClientMixin, ModbusProtocol):
             return socket.AF_INET
         return socket.AF_INET6
 
-    def connect(self):
+    def connect(self) -> bool:  # type: ignore[empty-body]
         """Connect to other end, overwritten."""
 
     def close(self):
