@@ -89,7 +89,28 @@ class AsyncModbusTlsClient(AsyncModbusTcpClient):
             self.comm_params.host,
             self.comm_params.port,
         )
-        return await self.transport_connect()
+        return await self.base_connect()
+
+    @classmethod
+    def generate_ssl(
+        cls,
+        certfile: str | None = None,
+        keyfile: str | None = None,
+        password: str | None = None,
+    ) -> ssl.SSLContext:
+        """Generate sslctx from cert/key/password.
+
+        :param certfile: Cert file path for TLS server request
+        :param keyfile: Key file path for TLS server request
+        :param password: Password for for decrypting private key file
+
+        Remark:
+        - MODBUS/TCP Security Protocol Specification demands TLSv2 at least
+        - verify_mode is set to ssl.NONE
+        """
+        return CommParams.generate_ssl(
+            False, certfile=certfile, keyfile=keyfile, password=password
+        )
 
 
 class ModbusTlsClient(ModbusTcpClient):
@@ -116,8 +137,6 @@ class ModbusTlsClient(ModbusTcpClient):
     :param timeout: Timeout for a request, in seconds.
     :param retries: Max number of retries per request.
     :param retry_on_empty: Retry on empty response.
-    :param close_comm_on_error: Close connection on error.
-    :param strict: Strict timing, 1.5 character between requests.
     :param broadcast_enable: True to treat id 0 as broadcast address.
     :param reconnect_delay: Minimum delay in seconds.milliseconds before reconnecting.
     :param reconnect_delay_max: Maximum delay in seconds.milliseconds before reconnecting.
@@ -160,7 +179,29 @@ class ModbusTlsClient(ModbusTcpClient):
         self.sslctx = CommParams.generate_ssl(
             False, certfile, keyfile, password, sslctx=sslctx
         )
-        self.params.server_hostname = server_hostname
+        self.server_hostname = server_hostname
+
+
+    @classmethod
+    def generate_ssl(
+        cls,
+        certfile: str | None = None,
+        keyfile: str | None = None,
+        password: str | None = None,
+    ) -> ssl.SSLContext:
+        """Generate sslctx from cert/key/password.
+
+        :param certfile: Cert file path for TLS server request
+        :param keyfile: Key file path for TLS server request
+        :param password: Password for for decrypting private key file
+
+        Remark:
+        - MODBUS/TCP Security Protocol Specification demands TLSv2 at least
+        - verify_mode is set to ssl.NONE
+        """
+        return CommParams.generate_ssl(
+            False, certfile=certfile, keyfile=keyfile, password=password,
+        )
 
     @property
     def connected(self) -> bool:
@@ -176,7 +217,7 @@ class ModbusTlsClient(ModbusTcpClient):
             if self.params.source_address:
                 sock.bind(self.params.source_address)
             self.socket = self.sslctx.wrap_socket(
-                sock, server_side=False, server_hostname=self.comm_params.host
+                sock, server_side=False, server_hostname=self.server_hostname
             )
             self.socket.settimeout(self.comm_params.timeout_connect)
             self.socket.connect((self.comm_params.host, self.comm_params.port))
