@@ -93,7 +93,7 @@ class AsyncModbusUdpClient(
             self.comm_params.host,
             self.comm_params.port,
         )
-        return await self.transport_connect()
+        return await self.base_connect()
 
 
 class ModbusUdpClient(ModbusBaseSyncClient):
@@ -114,8 +114,6 @@ class ModbusUdpClient(ModbusBaseSyncClient):
     :param timeout: Timeout for a request, in seconds.
     :param retries: Max number of retries per request.
     :param retry_on_empty: Retry on empty response.
-    :param close_comm_on_error: Close connection on error.
-    :param strict: Strict timing, 1.5 character between requests.
     :param broadcast_enable: True to treat id 0 as broadcast address.
     :param reconnect_delay: Minimum delay in seconds.milliseconds before reconnecting.
     :param reconnect_delay_max: Maximum delay in seconds.milliseconds before reconnecting.
@@ -139,6 +137,8 @@ class ModbusUdpClient(ModbusBaseSyncClient):
     Remark: There are no automatic reconnect as with AsyncModbusUdpClient
     """
 
+    socket: socket.socket | None
+
     def __init__(
         self,
         host: str,
