@@ -50,15 +50,15 @@ class RemoteSlaveContext(ModbusBaseSlaveContext):
     def setValues(self, fc_as_hex, address, values):
         """Set the datastore with the supplied values."""
         group_fx = self.decode(fc_as_hex)
-        if fc_as_hex in self._write_fc:
-            func_fc = self.__set_callbacks[f"{group_fx}{fc_as_hex}"]
-            if fc_as_hex in {0x0F, 0x10}:
-                self.result = func_fc(address, values)
-            else:
-                self.result = func_fc(address, values[0])
-        if self.result.isError():
-            return self.result
-        return None
+        if fc_as_hex not in self._write_fc:
+            raise ValueError(f"setValues() called with an non-write function code {fc_as_hex}")
+        func_fc = self.__set_callbacks[f"{group_fx}{fc_as_hex}"]
+        if fc_as_hex in {0x0F, 0x10}:  # Write Multiple Coils, Write Multiple Registers
+            self.result = func_fc(address, values)
+        else:
+            self.result = func_fc(address, values[0])
+        # if self.result.isError():
+        #    return self.result
 
     def __str__(self):
         """Return a string representation of the context.
@@ -73,42 +73,42 @@ class RemoteSlaveContext(ModbusBaseSlaveContext):
         if self.slave:
             kwargs["slave"] = self.slave
         self.__get_callbacks = {
-            "d": lambda a, c: self._client.read_discrete_inputs(  # pylint: disable=unnecessary-lambda
+            "d": lambda a, c: self._client.read_discrete_inputs(
                 a, c, **kwargs
             ),
-            "c": lambda a, c: self._client.read_coils(  # pylint: disable=unnecessary-lambda
+            "c": lambda a, c: self._client.read_coils(
                 a, c, **kwargs
             ),
-            "h": lambda a, c: self._client.read_holding_registers(  # pylint: disable=unnecessary-lambda
+            "h": lambda a, c: self._client.read_holding_registers(
                 a, c, **kwargs
             ),
-            "i": lambda a, c: self._client.read_input_registers(  # pylint: disable=unnecessary-lambda
+            "i": lambda a, c: self._client.read_input_registers(
                 a, c, **kwargs
             ),
         }
         self.__set_callbacks = {
-            "d5": lambda a, v: self._client.write_coil(  # pylint: disable=unnecessary-lambda
+            "d5": lambda a, v: self._client.write_coil(
                 a, v, **kwargs
             ),
-            "d15": lambda a, v: self._client.write_coils(  # pylint: disable=unnecessary-lambda
+            "d15": lambda a, v: self._client.write_coils(
                 a, v, **kwargs
             ),
-            "c5": lambda a, v: self._client.write_coil(  # pylint: disable=unnecessary-lambda
+            "c5": lambda a, v: self._client.write_coil(
                 a, v, **kwargs
             ),
-            "c15": lambda a, v: self._client.write_coils(  # pylint: disable=unnecessary-lambda
+            "c15": lambda a, v: self._client.write_coils(
                 a, v, **kwargs
             ),
-            "h6": lambda a, v: self._client.write_register(  # pylint: disable=unnecessary-lambda
+            "h6": lambda a, v: self._client.write_register(
                 a, v, **kwargs
             ),
-            "h16": lambda a, v: self._client.write_registers(  # pylint: disable=unnecessary-lambda
+            "h16": lambda a, v: self._client.write_registers(
                 a, v, **kwargs
             ),
-            "i6": lambda a, v: self._client.write_register(  # pylint: disable=unnecessary-lambda
+            "i6": lambda a, v: self._client.write_register(
                 a, v, **kwargs
             ),
-            "i16": lambda a, v: self._client.write_registers(  # pylint: disable=unnecessary-lambda
+            "i16": lambda a, v: self._client.write_registers(
                 a, v, **kwargs
             ),
         }
