@@ -7,6 +7,8 @@ import struct
 from datetime import datetime
 from typing import Any, Callable
 
+from pymodbus.datastore.context import ModbusBaseSlaveContext
+
 
 WORD_SIZE = 16
 
@@ -37,6 +39,18 @@ class Cell:
     count_write: int = 0
 
 
+class TextCell:  # pylint: disable=too-few-public-methods
+    """A textual representation of a single cell."""
+
+    type: str
+    access: str
+    value: str
+    action: str
+    action_kwargs: str
+    count_read: str
+    count_write: str
+
+
 @dataclasses.dataclass
 class Label:  # pylint: disable=too-many-instance-attributes
     """Defines all dict values.
@@ -94,8 +108,8 @@ class Setup:
     def __init__(self, runtime):
         """Initialize."""
         self.runtime = runtime
-        self.config = None
-        self.config_types = {
+        self.config = {}
+        self.config_types: dict[str, dict[str, Any]] = {
             Label.type_bits: {
                 Label.type: CellType.BITS,
                 Label.next: None,
@@ -358,7 +372,7 @@ class Setup:
             raise RuntimeError(f"INVALID key in setup: {self.config}")
 
 
-class ModbusSimulatorContext:
+class ModbusSimulatorContext(ModbusBaseSlaveContext):
     """Modbus simulator.
 
     :param config: A dict with structure as shown below.
@@ -420,7 +434,7 @@ class ModbusSimulatorContext:
             "write": [   --> allow write, efault is ReadOnly
                 [5, 5]  --> start, end bytes, repeated as needed
             ],
-            "bits": [  --> Define bits (1 register == 1 byte)
+            "bits": [  --> Define bits (1 register == 2 bytes)
                 [30, 31],  --> start, end registers, repeated as needed
                 {"addr": [32, 34], "value": 0xF1},  --> with value
                 {"addr": [35, 36], "action": "increment"},  --> with action
@@ -454,10 +468,10 @@ class ModbusSimulatorContext:
     start_time = int(datetime.now().timestamp())
 
     def __init__(
-        self, config: dict[str, Any], custom_actions: dict[str, Callable]
+        self, config: dict[str, Any], custom_actions: dict[str, Callable] | None
     ) -> None:
         """Initialize."""
-        self.registers: list[int] = []
+        self.registers: list[Cell] = []
         self.fc_offset: dict[int, int] = {}
         self.register_count = 0
         self.type_exception = False
@@ -474,7 +488,7 @@ class ModbusSimulatorContext:
     def get_text_register(self, register):
         """Get raw register."""
         reg = self.registers[register]
-        text_cell = Cell()
+        text_cell = TextCell()
         text_cell.type = self.registerType_id_to_name[reg.type]
         text_cell.access = str(reg.access)
         text_cell.count_read = str(reg.count_read)
@@ -587,8 +601,9 @@ class ModbusSimulatorContext:
             for i in range(real_address, real_address + reg_count):
                 reg = self.registers[i]
                 if reg.action:
+                    kwargs = reg.action_kwargs or {}
                     self.action_methods[reg.action](
-                        self.registers, i, reg, reg.action_kwargs
+                        self.registers, i, reg, **kwargs
                     )
                 self.registers[i].count_read += 1
                 while count and bit_index < 16:
@@ -736,11 +751,12 @@ class ModbusSimulatorContext:
     # Internal helper methods
     # --------------------------------------------
 
-    def validate_type(self, func_code, real_address, count):
+    def validate_type(self, func_code, real_address, count) -> bool:
         """Check if request is done against correct type.
 
         :meta private:
         """
+        check: tuple
         if func_code in self._bits_func_code:
             # Bit access
             check = (CellType.BITS, -1)
