@@ -17,7 +17,7 @@ class ModbusEvent:
 
         :raises NotImplementedException:
         """
-        raise NotImplementedException()
+        raise NotImplementedException
 
     def decode(self, event):
         """Decode the event message to its status bits.
@@ -25,7 +25,7 @@ class ModbusEvent:
         :param event: The event to decode
         :raises NotImplementedException:
         """
-        raise NotImplementedException()
+        raise NotImplementedException
 
 
 class RemoteReceiveEvent(ModbusEvent):
@@ -54,7 +54,7 @@ class RemoteReceiveEvent(ModbusEvent):
         self.listen = kwargs.get("listen", False)
         self.broadcast = kwargs.get("broadcast", False)
 
-    def encode(self):
+    def encode(self) -> bytes:
         """Encode the status bits to an event message.
 
         :returns: The encoded event message
@@ -64,7 +64,7 @@ class RemoteReceiveEvent(ModbusEvent):
         packet = pack_bitstring(bits)
         return packet
 
-    def decode(self, event):
+    def decode(self, event: bytes) -> None:
         """Decode the event message to its status bits.
 
         :param event: The event to decode
@@ -140,7 +140,7 @@ class RemoteSendEvent(ModbusEvent):
 
 
 class EnteredListenModeEvent(ModbusEvent):
-    """Enter Remote device Listen Only Mode
+    """Enter Remote device Listen Only Mode.
 
     The remote device stores this type of event byte when it enters
     the Listen Only Mode. The event is defined by a content of 04 hex.
