@@ -3,6 +3,16 @@
 Custom exceptions to be used in the Modbus code.
 """
 
+__all__ = [
+    "ModbusIOException",
+    "ParameterException",
+    "NotImplementedException",
+    "ConnectionException",
+    "NoSuchSlaveException",
+    "InvalidMessageReceivedException",
+    "MessageRegisterException",
+]
+
 
 class ModbusException(Exception):
     """Base modbus exception."""
@@ -13,13 +23,13 @@ class ModbusException(Exception):
         :param string: The message to append to the error
         """
         self.string = string
-        super().__init__()
+        super().__init__(string)
 
     def __str__(self):
         """Return string representation."""
         return f"Modbus Error: {self.string}"
 
-    def isError(self):  # pylint: disable=invalid-name
+    def isError(self):
         """Error"""
         return True
 
@@ -104,18 +114,3 @@ class MessageRegisterException(ModbusException):
         """Initialize."""
         message = f"[Error registering message] {string}"
         ModbusException.__init__(self, message)
-
-
-# --------------------------------------------------------------------------- #
-# Exported symbols
-# --------------------------------------------------------------------------- #
-__all__ = [
-    "ModbusException",
-    "ModbusIOException",
-    "ParameterException",
-    "NotImplementedException",
-    "ConnectionException",
-    "NoSuchSlaveException",
-    "InvalidMessageReceivedException",
-    "MessageRegisterException",
-]
