@@ -2,14 +2,24 @@
 
 Currently none of these messages are implemented
 """
+from __future__ import annotations
+
+
+__all__ = [
+    "FileRecord",
+    "ReadFileRecordRequest",
+    "ReadFileRecordResponse",
+    "WriteFileRecordRequest",
+    "WriteFileRecordResponse",
+    "ReadFifoQueueRequest",
+    "ReadFifoQueueResponse",
+]
+
 # pylint: disable=missing-type-doc
 import struct
 
-from pymodbus.pdu import (
-    ModbusExceptions as merror,
-    ModbusRequest,
-    ModbusResponse,
-)
+from pymodbus.pdu import ModbusExceptions as merror
+from pymodbus.pdu import ModbusRequest, ModbusResponse
 
 
 # ---------------------------------------------------------------------------#
@@ -21,7 +31,7 @@ class FileRecord:  # pylint: disable=eq-without-hash
     def __init__(self, **kwargs):
         """Initialize a new instance.
 
-        :params reference_type: Defaults to 0x06 (must be)
+        :params reference_type: must be 0x06
         :params file_number: Indicates which file number we are reading
         :params record_number: Indicates which record in the file
         :params record_data: The actual data of the record
@@ -87,6 +97,7 @@ class ReadFileRecordRequest(ModbusRequest):
     """
 
     function_code = 0x14
+    function_code_name = "read_file_record"
     _rtu_byte_count_pos = 2
 
     def __init__(self, records=None, **kwargs):
@@ -130,16 +141,15 @@ class ReadFileRecordRequest(ModbusRequest):
             if decoded[0] == 0x06:
                 self.records.append(record)
 
-    def execute(self, context):  # pylint: disable=unused-argument
+    def execute(self, _context):
         """Run a read exception status request against the store.
 
-        :param context: The datastore to request from
         :returns: The populated response
         """
         # TODO do some new context operation here # pylint: disable=fixme
         # if file number, record number, or address + length
         # is too big, return an error.
-        files = []
+        files: list[FileRecord] = []
         return ReadFileRecordResponse(files)
 
 
@@ -171,7 +181,7 @@ class ReadFileRecordResponse(ModbusResponse):
         total = sum(record.response_length + 1 for record in self.records)
         packet = struct.pack("B", total)
         for record in self.records:
-            packet += struct.pack(">BB", 0x06, record.record_length)
+            packet += struct.pack(">BB", record.record_length, 0x06)
             packet += record.record_data
         return packet
 
@@ -205,6 +215,7 @@ class WriteFileRecordRequest(ModbusRequest):
     """
 
     function_code = 0x15
+    function_code_name = "write_file_record"
     _rtu_byte_count_pos = 2
 
     def __init__(self, records=None, **kwargs):
@@ -254,10 +265,9 @@ class WriteFileRecordRequest(ModbusRequest):
             if decoded[0] == 0x06:
                 self.records.append(record)
 
-    def execute(self, context):  # pylint: disable=unused-argument
+    def execute(self, _context):
         """Run the write file record request against the context.
 
-        :param context: The datastore to request from
         :returns: The populated response
         """
         # TODO do some new context operation here # pylint: disable=fixme
@@ -334,6 +344,7 @@ class ReadFifoQueueRequest(ModbusRequest):
     """
 
     function_code = 0x18
+    function_code_name = "read_fifo_queue"
     _rtu_frame_size = 6
 
     def __init__(self, address=0x0000, **kwargs):
@@ -359,10 +370,9 @@ class ReadFifoQueueRequest(ModbusRequest):
         """
         self.address = struct.unpack(">H", data)[0]
 
-    def execute(self, context):  # pylint: disable=unused-argument
+    def execute(self, _context):
         """Run a read exception status request against the store.
 
-        :param context: The datastore to request from
         :returns: The populated response
         """
         if not 0x0000 <= self.address <= 0xFFFF:
@@ -427,17 +437,3 @@ class ReadFifoQueueResponse(ModbusResponse):
         for index in range(0, count - 4):
             idx = 4 + index * 2
             self.values.append(struct.unpack(">H", data[idx : idx + 2])[0])
-
-
-# ---------------------------------------------------------------------------#
-#  Exported symbols
-# ---------------------------------------------------------------------------#
-__all__ = [
-    "FileRecord",
-    "ReadFileRecordRequest",
-    "ReadFileRecordResponse",
-    "WriteFileRecordRequest",
-    "WriteFileRecordResponse",
-    "ReadFifoQueueRequest",
-    "ReadFifoQueueResponse",
-]
