@@ -164,6 +164,7 @@ class ModbusExceptions:  # pylint: disable=too-few-public-methods
     SlaveFailure = 0x04
     Acknowledge = 0x05
     SlaveBusy = 0x06
+    NegativeAcknowledge = 0x07
     MemoryParityError = 0x08
     GatewayPathUnavailable = 0x0A
     GatewayNoResponse = 0x0B
@@ -248,7 +249,10 @@ class IllegalFunctionRequest(ModbusRequest):
     def decode(self, _data):
         """Decode so this failure will run correctly."""
 
-    def execute(self, _context):
+    def encode(self):
+        """Decode so this failure will run correctly."""
+
+    async def execute(self, _context):
         """Build an illegal function request error response.
 
         :returns: The error response packet
