@@ -1,13 +1,20 @@
 """Register Reading Request/Response."""
+
+__all__ = [
+    "ReadHoldingRegistersRequest",
+    "ReadHoldingRegistersResponse",
+    "ReadInputRegistersRequest",
+    "ReadInputRegistersResponse",
+    "ReadRegistersResponseBase",
+    "ReadWriteMultipleRegistersRequest",
+    "ReadWriteMultipleRegistersResponse",
+]
+
 # pylint: disable=missing-type-doc
 import struct
 
-from pymodbus.constants import Defaults
-from pymodbus.pdu import (
-    ModbusExceptions as merror,
-    ModbusRequest,
-    ModbusResponse,
-)
+from pymodbus.pdu import ModbusExceptions as merror
+from pymodbus.pdu import ModbusRequest, ModbusResponse
 
 
 class ReadRegistersRequestBase(ModbusRequest):
@@ -15,14 +22,14 @@ class ReadRegistersRequestBase(ModbusRequest):
 
     _rtu_frame_size = 8
 
-    def __init__(self, address, count, unit=Defaults.Slave, **kwargs):
+    def __init__(self, address, count, slave=0, **kwargs):
         """Initialize a new instance.
 
         :param address: The address to start the read from
         :param count: The number of registers to read
-        :param unit: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         """
-        super().__init__(unit, **kwargs)
+        super().__init__(slave, **kwargs)
         self.address = address
         self.count = count
 
@@ -52,7 +59,7 @@ class ReadRegistersRequestBase(ModbusRequest):
 
         :returns: A string representation of the instance
         """
-        return f"ReadRegisterRequest ({self.address},{self.count})"
+        return f"{self.__class__.__name__} ({self.address},{self.count})"
 
 
 class ReadRegistersResponseBase(ModbusResponse):
@@ -63,13 +70,13 @@ class ReadRegistersResponseBase(ModbusResponse):
 
     _rtu_byte_count_pos = 2
 
-    def __init__(self, values, unit=Defaults.Slave, **kwargs):
+    def __init__(self, values, slave=0, **kwargs):
         """Initialize a new instance.
 
         :param values: The values to write to
-        :param unit: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         """
-        super().__init__(unit, **kwargs)
+        super().__init__(slave, **kwargs)
 
         #: A list of register values
         self.registers = values or []
@@ -94,7 +101,7 @@ class ReadRegistersResponseBase(ModbusResponse):
         for i in range(1, byte_count + 1, 2):
             self.registers.append(struct.unpack(">H", data[i : i + 2])[0])
 
-    def getRegister(self, index):  # pylint: disable=(invalid-name
+    def getRegister(self, index):
         """Get the requested register.
 
         :param index: The indexed register to retrieve
@@ -121,27 +128,30 @@ class ReadHoldingRegistersRequest(ReadRegistersRequestBase):
     """
 
     function_code = 3
+    function_code_name = "read_holding_registers"
 
-    def __init__(self, address=None, count=None, unit=Defaults.Slave, **kwargs):
+    def __init__(self, address=None, count=None, slave=0, **kwargs):
         """Initialize a new instance of the request.
 
         :param address: The starting address to read from
         :param count: The number of registers to read from address
-        :param unit: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         """
-        super().__init__(address, count, unit, **kwargs)
+        super().__init__(address, count, slave, **kwargs)
 
-    def execute(self, context):
+    async def execute(self, context):
         """Run a read holding request against a datastore.
 
         :param context: The datastore to request from
-        :returns: An initialized :py:class:`~pymodbus.register_read_message.ReadHoldingRegistersResponse`, or an :py:class:`~pymodbus.pdu.ExceptionResponse` if an error occurred
+        :returns: An initialized :py:class:`~pymodbus.register_read_message.ReadHoldingRegistersResponse`
         """
         if not (1 <= self.count <= 0x7D):
             return self.doException(merror.IllegalValue)
         if not context.validate(self.function_code, self.address, self.count):
             return self.doException(merror.IllegalAddress)
-        values = context.getValues(self.function_code, self.address, self.count)
+        values = await context.async_getValues(
+            self.function_code, self.address, self.count
+        )
         return ReadHoldingRegistersResponse(values)
 
 
@@ -178,27 +188,30 @@ class ReadInputRegistersRequest(ReadRegistersRequestBase):
     """
 
     function_code = 4
+    function_code_name = "read_input_registers"
 
-    def __init__(self, address=None, count=None, unit=Defaults.Slave, **kwargs):
+    def __init__(self, address=None, count=None, slave=0, **kwargs):
         """Initialize a new instance of the request.
 
         :param address: The starting address to read from
         :param count: The number of registers to read from address
-        :param unit: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         """
-        super().__init__(address, count, unit, **kwargs)
+        super().__init__(address, count, slave, **kwargs)
 
-    def execute(self, context):
+    async def execute(self, context):
         """Run a read input request against a datastore.
 
         :param context: The datastore to request from
-        :returns: An initialized :py:class:`~pymodbus.register_read_message.ReadInputRegistersResponse`, or an :py:class:`~pymodbus.pdu.ExceptionResponse` if an error occurred
+        :returns: An initialized :py:class:`~pymodbus.register_read_message.ReadInputRegistersResponse`
         """
         if not (1 <= self.count <= 0x7D):
             return self.doException(merror.IllegalValue)
         if not context.validate(self.function_code, self.address, self.count):
             return self.doException(merror.IllegalAddress)
-        values = context.getValues(self.function_code, self.address, self.count)
+        values = await context.async_getValues(
+            self.function_code, self.address, self.count
+        )
         return ReadInputRegistersResponse(values)
 
 
@@ -241,6 +254,7 @@ class ReadWriteMultipleRegistersRequest(ModbusRequest):
     """
 
     function_code = 23
+    function_code_name = "read_write_multiple_registers"
     _rtu_byte_count_pos = 10
 
     def __init__(self, **kwargs):
@@ -295,11 +309,11 @@ class ReadWriteMultipleRegistersRequest(ModbusRequest):
             register = struct.unpack(">H", data[i : i + 2])[0]
             self.write_registers.append(register)
 
-    def execute(self, context):
+    async def execute(self, context):
         """Run a write single register request against a datastore.
 
         :param context: The datastore to request from
-        :returns: An initialized :py:class:`~pymodbus.register_read_message.ReadWriteMultipleRegistersResponse`, or an :py:class:`~pymodbus.pdu.ExceptionResponse` if an error occurred
+        :returns: An initialized :py:class:`~pymodbus.register_read_message.ReadWriteMultipleRegistersResponse`
         """
         if not (1 <= self.read_count <= 0x07D):
             return self.doException(merror.IllegalValue)
@@ -313,8 +327,10 @@ class ReadWriteMultipleRegistersRequest(ModbusRequest):
             return self.doException(merror.IllegalAddress)
         if not context.validate(self.function_code, self.read_address, self.read_count):
             return self.doException(merror.IllegalAddress)
-        context.setValues(self.function_code, self.write_address, self.write_registers)
-        registers = context.getValues(
+        await context.async_setValues(
+            self.function_code, self.write_address, self.write_registers
+        )
+        registers = await context.async_getValues(
             self.function_code, self.read_address, self.read_count
         )
         return ReadWriteMultipleRegistersResponse(registers)
@@ -390,17 +406,3 @@ class ReadWriteMultipleRegistersResponse(ModbusResponse):
         :returns: A string representation of the instance
         """
         return f"ReadWriteNRegisterResponse ({len(self.registers)})"
-
-
-# ---------------------------------------------------------------------------#
-#  Exported symbols
-# ---------------------------------------------------------------------------#
-__all__ = [
-    "ReadHoldingRegistersRequest",
-    "ReadHoldingRegistersResponse",
-    "ReadInputRegistersRequest",
-    "ReadInputRegistersResponse",
-    "ReadRegistersResponseBase",
-    "ReadWriteMultipleRegistersRequest",
-    "ReadWriteMultipleRegistersResponse",
-]
