@@ -1,12 +1,19 @@
 """Register Writing Request/Response Messages."""
+
+__all__ = [
+    "WriteSingleRegisterRequest",
+    "WriteSingleRegisterResponse",
+    "WriteMultipleRegistersRequest",
+    "WriteMultipleRegistersResponse",
+    "MaskWriteRegisterRequest",
+    "MaskWriteRegisterResponse",
+]
+
 # pylint: disable=missing-type-doc
 import struct
 
-from pymodbus.pdu import (
-    ModbusExceptions as merror,
-    ModbusRequest,
-    ModbusResponse,
-)
+from pymodbus.pdu import ModbusExceptions as merror
+from pymodbus.pdu import ModbusRequest, ModbusResponse
 
 
 class WriteSingleRegisterRequest(ModbusRequest):
@@ -18,15 +25,16 @@ class WriteSingleRegisterRequest(ModbusRequest):
     """
 
     function_code = 6
+    function_code_name = "write_register"
     _rtu_frame_size = 8
 
-    def __init__(self, address=None, value=None, unit=None, **kwargs):
+    def __init__(self, address=None, value=None, slave=None, **kwargs):
         """Initialize a new instance.
 
         :param address: The address to start writing add
         :param value: The values to write
         """
-        super().__init__(unit=unit, **kwargs)
+        super().__init__(slave=slave, **kwargs)
         self.address = address
         self.value = value
 
@@ -49,7 +57,7 @@ class WriteSingleRegisterRequest(ModbusRequest):
         """
         self.address, self.value = struct.unpack(">HH", data)
 
-    def execute(self, context):
+    async def execute(self, context):
         """Run a write single register request against a datastore.
 
         :param context: The datastore to request from
@@ -60,8 +68,10 @@ class WriteSingleRegisterRequest(ModbusRequest):
         if not context.validate(self.function_code, self.address, 1):
             return self.doException(merror.IllegalAddress)
 
-        context.setValues(self.function_code, self.address, [self.value])
-        values = context.getValues(self.function_code, self.address, 1)
+        await context.async_setValues(
+            self.function_code, self.address, [self.value]
+        )
+        values = await context.async_getValues(self.function_code, self.address, 1)
         return WriteSingleRegisterResponse(self.address, values[0])
 
     def get_response_pdu_size(self):
@@ -146,16 +156,17 @@ class WriteMultipleRegistersRequest(ModbusRequest):
     """
 
     function_code = 16
+    function_code_name = "write_registers"
     _rtu_byte_count_pos = 6
     _pdu_length = 5  # func + adress1 + adress2 + outputQuant1 + outputQuant2
 
-    def __init__(self, address=None, values=None, unit=None, **kwargs):
+    def __init__(self, address=None, values=None, slave=None, **kwargs):
         """Initialize a new instance.
 
         :param address: The address to start writing to
         :param values: The values to write
         """
-        super().__init__(unit=unit, **kwargs)
+        super().__init__(slave=slave, **kwargs)
         self.address = address
         if values is None:
             values = []
@@ -189,7 +200,7 @@ class WriteMultipleRegistersRequest(ModbusRequest):
         for idx in range(5, (self.count * 2) + 5, 2):
             self.values.append(struct.unpack(">H", data[idx : idx + 2])[0])
 
-    def execute(self, context):
+    async def execute(self, context):
         """Run a write single register request against a datastore.
 
         :param context: The datastore to request from
@@ -202,7 +213,9 @@ class WriteMultipleRegistersRequest(ModbusRequest):
         if not context.validate(self.function_code, self.address, self.count):
             return self.doException(merror.IllegalAddress)
 
-        context.setValues(self.function_code, self.address, self.values)
+        await context.async_setValues(
+            self.function_code, self.address, self.values
+        )
         return WriteMultipleRegistersResponse(self.address, self.count)
 
     def get_response_pdu_size(self):
@@ -274,11 +287,12 @@ class MaskWriteRegisterRequest(ModbusRequest):
     """This function code is used to modify the contents.
 
     Of a specified holding register using a combination of an AND mask,
-    an OR mask, and the register"s current contents.
+    an OR mask, and the register's current contents.
     The function can be used to set or clear individual bits in the register.
     """
 
     function_code = 0x16
+    function_code_name = "mask_write_register"
     _rtu_frame_size = 10
 
     def __init__(self, address=0x0000, and_mask=0xFFFF, or_mask=0x0000, **kwargs):
@@ -307,7 +321,7 @@ class MaskWriteRegisterRequest(ModbusRequest):
         """
         self.address, self.and_mask, self.or_mask = struct.unpack(">HHH", data)
 
-    def execute(self, context):
+    async def execute(self, context):
         """Run a mask write register request against the store.
 
         :param context: The datastore to request from
@@ -319,9 +333,11 @@ class MaskWriteRegisterRequest(ModbusRequest):
             return self.doException(merror.IllegalValue)
         if not context.validate(self.function_code, self.address, 1):
             return self.doException(merror.IllegalAddress)
-        values = context.getValues(self.function_code, self.address, 1)[0]
+        values = (await context.async_getValues(self.function_code, self.address, 1))[0]
         values = (values & self.and_mask) | (self.or_mask & ~self.and_mask)
-        context.setValues(self.function_code, self.address, [values])
+        await context.async_setValues(
+            self.function_code, self.address, [values]
+        )
         return MaskWriteRegisterResponse(self.address, self.and_mask, self.or_mask)
 
 
@@ -359,16 +375,3 @@ class MaskWriteRegisterResponse(ModbusResponse):
         :param data: The packet data to decode
         """
         self.address, self.and_mask, self.or_mask = struct.unpack(">HHH", data)
-
-
-# ---------------------------------------------------------------------------#
-#  Exported symbols
-# ---------------------------------------------------------------------------#
-__all__ = [
-    "WriteSingleRegisterRequest",
-    "WriteSingleRegisterResponse",
-    "WriteMultipleRegistersRequest",
-    "WriteMultipleRegistersResponse",
-    "MaskWriteRegisterRequest",
-    "MaskWriteRegisterResponse",
-]
