@@ -7,7 +7,8 @@ import dataclasses
 import importlib
 import json
 import os
-from time import time
+from time import sleep
+from typing import TYPE_CHECKING
 
 
 try:
@@ -16,6 +17,9 @@ try:
     AIOHTTP_MISSING = False
 except ImportError:
     AIOHTTP_MISSING = True
+    if TYPE_CHECKING:  # always False at runtime
+        # type checkers do not understand the Raise RuntimeError in __init__()
+        from aiohttp import web
 
 from pymodbus.datastore import ModbusServerContext, ModbusSimulatorContext
 from pymodbus.datastore.simulator import Label
@@ -160,7 +164,7 @@ class ModbusSimulatorServer:
         self.modbus_server = comm(framer=framer, context=datastore, **server)
         self.serving: asyncio.Future = asyncio.Future()
         self.log_file = log_file
-        self.site = None
+        self.site: web.TCPSite | None = None
         self.runner: web.AppRunner
         self.http_host = http_host
         self.http_port = http_port
@@ -176,13 +180,13 @@ class ModbusSimulatorServer:
         )
         self.web_app.on_startup.append(self.start_modbus_server)
         self.web_app.on_shutdown.append(self.stop_modbus_server)
-        self.generator_html = {
+        self.generator_html: dict[str, list] = {
             "log": ["", self.build_html_log],
             "registers": ["", self.build_html_registers],
             "calls": ["", self.build_html_calls],
             "server": ["", self.build_html_server],
         }
-        self.generator_json = {
+        self.generator_json: dict[str, list] = {
             "log_json": [None, self.build_json_log],
             "registers_json": [None, self.build_json_registers],
             "calls_json": [None, self.build_json_calls],
@@ -203,20 +207,21 @@ class ModbusSimulatorServer:
                 self.generator_html[entry][0] = handle.read()
         self.refresh_rate = 0
         self.register_filter: list[int] = []
-        self.call_list: list[tuple] = []
+        self.call_list: list[CallTracer] = []
         self.request_lookup = ServerDecoder.getFCdict()
         self.call_monitor = CallTypeMonitor()
         self.call_response = CallTypeResponse()
+        self.api_key: web.AppKey = web.AppKey("modbus_server")
 
     async def start_modbus_server(self, app):
         """Start Modbus server as asyncio task."""
         try:
             if getattr(self.modbus_server, "start", None):
                 await self.modbus_server.start()
-            app["modbus_server"] = asyncio.create_task(
+            app[self.api_key] = asyncio.create_task(
                 self.modbus_server.serve_forever()
             )
-            app["modbus_server"].set_name("simulator modbus server")
+            app[self.api_key].set_name("simulator modbus server")
         except Exception as exc:
             Log.error("Error starting modbus server, reason: {}", exc)
             raise exc
@@ -228,9 +233,9 @@ class ModbusSimulatorServer:
         """Stop modbus server."""
         Log.info("Stopping modbus server")
         await self.modbus_server.shutdown()
-        app["modbus_server"].cancel()
+        app[self.api_key].cancel()
         with contextlib.suppress(asyncio.exceptions.CancelledError):
-            await app["modbus_server"]
+            await app[self.api_key]
 
         Log.info("Modbus server Stopped")
 
@@ -255,7 +260,7 @@ class ModbusSimulatorServer:
         self.site = None
         if not self.serving.done():
             self.serving.set_result(True)
-        await asyncio.sleep(0.1)
+        await asyncio.sleep(0)
 
     async def handle_html_static(self, request):
         """Handle static html."""
@@ -339,7 +344,7 @@ class ModbusSimulatorServer:
         )
         return new_html
 
-    def build_html_calls(self, params, html):
+    def build_html_calls(self, params: dict, html: str) -> str:
         """Build html calls page."""
         result_txt, foot = self.helper_build_html_submit(params)
         if not foot:
@@ -375,10 +380,10 @@ class ModbusSimulatorServer:
         for function in self.request_lookup.values():
             selected = (
                 "selected"
-                if function.function_code == self.call_monitor.function
+                if function.function_code == self.call_monitor.function  #type: ignore[attr-defined]
                 else ""
             )
-            function_codes += f"<option value={function.function_code} {selected}>{function.function_code_name}</option>"
+            function_codes += f"<option value={function.function_code} {selected}>{function.function_code_name}</option>"  #type: ignore[attr-defined]
         simulation_action = (
             "ACTIVE" if self.call_response.active != RESPONSE_INACTIVE else ""
         )
@@ -389,7 +394,7 @@ class ModbusSimulatorServer:
         call_rows = ""
         for entry in reversed(self.call_list):
             # req_obj = self.request_lookup[entry[1]]
-            call_rows += f"<tr><td>{entry.call} - {entry.fc}</td><td>{entry.address}</td><td>{entry.count}</td><td>{entry.data}</td></tr>"
+            call_rows += f"<tr><td>{entry.call} - {entry.fc}</td><td>{entry.address}</td><td>{entry.count}</td><td>{entry.data.decode()}</td></tr>"
             # line += req_obj.funcion_code_name
         new_html = (
             html.replace("<!--SIMULATION_ACTIVE-->", simulation_action)
@@ -530,7 +535,7 @@ class ModbusSimulatorServer:
         self.call_monitor.range_start = range_start
         self.call_monitor.range_stop = range_stop
         self.call_monitor.function = (
-            int(params["function"]) if params["function"] else ""
+            int(params["function"]) if params["function"] else -1
         )
         self.call_monitor.hex = "show_hex" in params
         self.call_monitor.decode = "show_decode" in params
@@ -598,7 +603,7 @@ class ModbusSimulatorServer:
                 fc=response.function_code,
                 address=response.address if hasattr(response, "address") else -1,
                 count=response.count if hasattr(response, "count") else -1,
-                data="-",
+                data=b"-",
             )
             self.call_list.append(tracer)
             self.call_monitor.trace_response = False
@@ -616,7 +621,7 @@ class ModbusSimulatorServer:
                     "Delaying response by {}s for all incoming requests",
                     self.call_response.delay,
                 )
-                time.sleep(self.call_response.delay)  # change to async
+                sleep(self.call_response.delay)  # change to async
             else:
                 pass
                 # self.call_response.change_rate
@@ -645,7 +650,7 @@ class ModbusSimulatorServer:
         """
         if self.call_monitor.function not in {-1, request.function_code}:
             return
-        address = (request.address if hasattr(request, "address") else -1,)
+        address = request.address if hasattr(request, "address") else -1
         if self.call_monitor.range_start != -1 and address != -1:
             if (
                 self.call_monitor.range_start > address
@@ -657,7 +662,7 @@ class ModbusSimulatorServer:
             fc=request.function_code,
             address=address,
             count=request.count if hasattr(request, "count") else -1,
-            data="-",
+            data=b"-",
         )
         self.call_list.append(tracer)
         self.call_monitor.trace_response = True
