@@ -2,6 +2,7 @@
 
 __all__ = [
     "DictTransactionManager",
+    "ModbusTransactionManager",
     "ModbusSocketFramer",
     "ModbusTlsFramer",
     "ModbusRtuFramer",
@@ -17,9 +18,9 @@ from functools import partial
 from threading import RLock
 
 from pymodbus.exceptions import (
+    ConnectionException,
     InvalidMessageReceivedException,
     ModbusIOException,
-    NotImplementedException,
 )
 from pymodbus.framer.ascii_framer import ModbusAsciiFramer
 from pymodbus.framer.binary_framer import ModbusBinaryFramer
@@ -47,6 +48,8 @@ class ModbusTransactionManager:
         while (count < 3)
 
     This module helps to abstract this away from the framer and protocol.
+
+    Results are keyed based on the supplied transaction id.
     """
 
     def __init__(self, client, **kwargs):
@@ -62,11 +65,19 @@ class ModbusTransactionManager:
         self.retry_on_empty = kwargs.get("retry_on_empty", False)
         self.retry_on_invalid = kwargs.get("retry_on_invalid", False)
         self.retries = kwargs.get("retries", 3)
+        self.transactions = {}
         self._transaction_lock = RLock()
         self._no_response_devices = []
         if client:
             self._set_adu_size()
 
+    def __iter__(self):
+        """Iterate over the current managed transactions.
+
+        :returns: An iterator of the managed transactions
+        """
+        return iter(self.transactions.keys())
+
     def _set_adu_size(self):
         """Set adu size."""
         # base ADU size of modbus frame in bytes
@@ -212,6 +223,7 @@ class ModbusTransactionManager:
                         else:
                             break
                         # full = False
+                        Log.debug("Retry getting response: - {}", _buffer)
                     addTransaction = partial(  # pylint: disable=invalid-name
                         self.addTransaction,
                         tid=request.transaction_id,
@@ -231,7 +243,7 @@ class ModbusTransactionManager:
                                 "/Unable to decode response"
                             )
                             response = ModbusIOException(
-                                last_exception, request.function_code
+                                last_exception, request.function_code  # type: ignore[assignment]
                             )
                         self.client.close()
                     if hasattr(self.client, "state"):
@@ -315,7 +327,7 @@ class ModbusTransactionManager:
             result = self._recv(response_length, full)
             # result2 = self._recv(response_length, full)
             Log.debug("RECV: {}", result, ":hex")
-        except (OSError, ModbusIOException, InvalidMessageReceivedException) as msg:
+        except (OSError, ModbusIOException, InvalidMessageReceivedException, ConnectionException) as msg:
             self.client.close()
             Log.debug("Transaction failed. ({}) ", msg)
             last_exception = msg
@@ -376,7 +388,7 @@ class ModbusTransactionManager:
                             IndexError  # response length indeterminate with available bytes
                         ):
                             expected_response_length = (
-                                self.client.framer.get_expected_response_length(
+                                self._get_expected_response_length(
                                     read_min
                                 )
                             )
@@ -414,82 +426,23 @@ class ModbusTransactionManager:
             self.client.state = ModbusTransactionState.PROCESSING_REPLY
         return result
 
+    def _get_expected_response_length(self, data):
+        """Get the expected response length.
+
+        :param data: Message data read so far
+        :raises IndexError: If not enough data to read byte count
+        :return: Total frame size
+        """
+        func_code = int(data[1])
+        pdu_class = self.client.framer.decoder.lookupPduClass(func_code)
+        return pdu_class.calculateRtuFrameSize(data)
+
     def addTransaction(self, request, tid=None):
         """Add a transaction to the handler.
 
         This holds the request in case it needs to be resent.
         After being sent, the request is removed.
 
-        :param request: The request to hold on to
-        :param tid: The overloaded transaction id to use
-        :raises NotImplementedException:
-        """
-        raise NotImplementedException("addTransaction")
-
-    def getTransaction(self, tid):
-        """Return a transaction matching the referenced tid.
-
-        If the transaction does not exist, None is returned
-
-        :param tid: The transaction to retrieve
-        :raises NotImplementedException:
-        """
-        raise NotImplementedException("getTransaction")
-
-    def delTransaction(self, tid):
-        """Remove a transaction matching the referenced tid.
-
-        :param tid: The transaction to remove
-        :raises NotImplementedException:
-        """
-        raise NotImplementedException("delTransaction")
-
-    def getNextTID(self):
-        """Retrieve the next unique transaction identifier.
-
-        This handles incrementing the identifier after
-        retrieval
-
-        :returns: The next unique transaction identifier
-        """
-        self.tid = (self.tid + 1) & 0xFFFF
-        return self.tid
-
-    def reset(self):
-        """Reset the transaction identifier."""
-        self.tid = 0
-        self.transactions = type(  # pylint: disable=attribute-defined-outside-init
-            self.transactions
-        )()
-
-
-class DictTransactionManager(ModbusTransactionManager):
-    """Implements a transaction for a manager.
-
-    Where the results are keyed based on the supplied transaction id.
-    """
-
-    def __init__(self, client, **kwargs):
-        """Initialize an instance of the ModbusTransactionManager.
-
-        :param client: The client socket wrapper
-        """
-        self.transactions = {}
-        super().__init__(client, **kwargs)
-
-    def __iter__(self):
-        """Iterate over the current managed transactions.
-
-        :returns: An iterator of the managed transactions
-        """
-        return iter(self.transactions.keys())
-
-    def addTransaction(self, request, tid=None):
-        """Add a transaction to the handler.
-
-        This holds the requests in case it needs to be resent.
-        After being sent, the request is removed.
-
         :param request: The request to hold on to
         :param tid: The overloaded transaction id to use
         """
@@ -508,7 +461,9 @@ class DictTransactionManager(ModbusTransactionManager):
         Log.debug("Getting transaction {}", tid)
         if not tid:
             if self.transactions:
-                return self.transactions.popitem()[1]
+                ret = self.transactions.popitem()[1]
+                self.transactions.clear()
+                return ret
             return None
         return self.transactions.pop(tid, None)
 
@@ -519,3 +474,25 @@ class DictTransactionManager(ModbusTransactionManager):
         """
         Log.debug("deleting transaction {}", tid)
         self.transactions.pop(tid, None)
+
+    def getNextTID(self):
+        """Retrieve the next unique transaction identifier.
+
+        This handles incrementing the identifier after
+        retrieval
+
+        :returns: The next unique transaction identifier
+        """
+        if self.tid < 65000:
+            self.tid += 1
+        else:
+            self.tid = 1
+        return self.tid
+
+    def reset(self):
+        """Reset the transaction identifier."""
+        self.tid = 0
+        self.transactions = {}
+
+class DictTransactionManager(ModbusTransactionManager):
+    """Old alias for ModbusTransactionManager."""
