@@ -20,6 +20,7 @@ from pymodbus.client.mixin import ModbusClientMixin
 from pymodbus.datastore import ModbusSlaveContext
 from pymodbus.datastore.store import ModbusSequentialDataBlock
 from pymodbus.exceptions import ConnectionException, ModbusException, ModbusIOException
+from pymodbus.pdu import ModbusRequest
 from pymodbus.transport import CommType
 
 
@@ -121,7 +122,6 @@ def test_client_mixin(arglist, method, arg, pdu_request):
                     "timeout": 3 + 2,
                     "retries": 3 + 2,
                     "retry_on_empty": True,
-                    "close_comm_on_error": True,
                     "strict": False,
                     "broadcast_enable": not False,
                     "reconnect_delay": 117,
@@ -131,7 +131,6 @@ def test_client_mixin(arglist, method, arg, pdu_request):
                     "timeout": 3,
                     "retries": 3,
                     "retry_on_empty": False,
-                    "close_comm_on_error": False,
                     "strict": True,
                     "broadcast_enable": False,
                     "reconnect_delay": 100,
@@ -291,8 +290,11 @@ async def test_client_connection_made():
     """Test protocol made connection."""
     client = lib_client.AsyncModbusTcpClient("127.0.0.1")
     assert not client.connected
-    client.connection_made(mock.AsyncMock())
-    assert client.connected
+
+    transport = mock.AsyncMock()
+    transport.close = lambda : ()
+    client.connection_made(transport)
+    # assert await client.connected
     client.close()
 
 
@@ -389,7 +391,7 @@ class MockTransport:
     async def delayed_resp(self):
         """Send a response to a received packet."""
         await asyncio.sleep(0.05)
-        resp = self.req.execute(self.ctx)
+        resp = await self.req.execute(self.ctx)
         pkt = self.base.framer.buildPacket(resp)
         self.base.data_received(pkt)
 
@@ -425,8 +427,7 @@ async def test_client_execute_broadcast():
     transport = MockTransport(base, request)
     base.connection_made(transport=transport)
 
-    response = await base.async_execute(request)
-    assert  response == b'Broadcast write sent - no response expected'
+    assert not await base.async_execute(request)
 
 async def test_client_protocol_retry():
     """Test the client protocol execute method with retries."""
@@ -575,12 +576,46 @@ def test_client_tls_connect2():
 )
 def test_client_mixin_convert(datatype, registers, value):
     """Test converter methods."""
-    regs = ModbusClientMixin.convert_to_registers(value, datatype)
-    result = ModbusClientMixin.convert_from_registers(regs, datatype)
+    result = ModbusClientMixin.convert_from_registers(registers, datatype)
     if datatype == ModbusClientMixin.DATATYPE.FLOAT32:
         result = round(result, 6)
-    assert regs == registers
     assert result == value
+    regs = ModbusClientMixin.convert_to_registers(value, datatype)
+    assert regs == registers
+
+
+@pytest.mark.parametrize(
+    ("datatype", "value", "registers"),
+    [
+        (ModbusClientMixin.DATATYPE.STRING, "0123", [b'\x30\x31', b'\x32\x33']),
+        (ModbusClientMixin.DATATYPE.UINT16, 258, [b'\x01\x02']),
+        (ModbusClientMixin.DATATYPE.INT16, -32510, [b'\x81\x02']),
+        (ModbusClientMixin.DATATYPE.UINT32, 16909060, [b'\x01\x02', b'\x03\x04']),
+        (ModbusClientMixin.DATATYPE.INT32, -2130574588, [b'\x81\x02', b'\x03\x04']),
+        (
+            ModbusClientMixin.DATATYPE.UINT64,
+            72623859790382856,
+            [b'\x01\x02', b'\x03\x04', b'\x05\x06', b'\x07\x08'],
+        ),
+        (
+            ModbusClientMixin.DATATYPE.INT64,
+            -9150748177064392952,
+            [b'\x81\x02', b'\x03\x04', b'\x05\x06', b'\x07\x08'],
+        ),
+        (ModbusClientMixin.DATATYPE.FLOAT32, 8.125736, [b'\x41\x02', b'\x03\x04']),
+        (ModbusClientMixin.DATATYPE.FLOAT64, 147552.502453, [b'\x41\x02', b'\x03\x04', b'\x05\x06', b'\x14\x16']),
+    ],
+)
+def test_client_mixin_convert_1234(datatype, registers, value):
+    """Test converter methods."""
+    for i in range(0, len(registers)):
+        registers[i] = int.from_bytes(registers[i], "big")
+    regs = ModbusClientMixin.convert_to_registers(value, datatype)
+    result = ModbusClientMixin.convert_from_registers(regs, datatype)
+    if datatype == ModbusClientMixin.DATATYPE.FLOAT32 or datatype == ModbusClientMixin.DATATYPE.FLOAT64:
+        result = round(result, 6)
+    assert result == value
+    assert regs == registers
 
 
 def test_client_mixin_convert_fail():
@@ -598,7 +633,11 @@ async def test_client_build_response():
     with pytest.raises(ConnectionException):
         await client.build_response(0)
 
+
 async def test_client_mixin_execute():
-    """Test dummy execute."""
+    """Test dummy execute for both sync and async."""
     client = ModbusClientMixin()
-    assert client.execute(None)
+    with pytest.raises(NotImplementedError):
+        client.execute(ModbusRequest())
+    with pytest.raises(NotImplementedError):
+        await client.execute(ModbusRequest())
