@@ -1,4 +1,5 @@
 """Test datastore."""
+
 import asyncio
 import copy
 import json
@@ -9,6 +10,7 @@ import pytest
 from pymodbus.datastore import ModbusSimulatorContext
 from pymodbus.datastore.simulator import Cell, CellType, Label
 from pymodbus.server import ModbusSimulatorServer
+from pymodbus.transport import NULLMODEM_HOST
 
 
 FX_READ_BIT = 1
@@ -101,7 +103,7 @@ class TestSimulator:
     default_server_config = {
         "server": {
             "comm": "tcp",
-            "host": "0.0.0.0",
+            "host": NULLMODEM_HOST,
             "port": 5020,
             "ignore_missing_slaves": False,
             "framer": "socket",
@@ -526,7 +528,12 @@ class TestSimulator:
         exc_simulator.registers[30].value = regs[0]
         exc_simulator.registers[31].value = regs[1]
         for expect_value in expected:
-            regs = exc_simulator.getValues(FX_READ_REG, 30, reg_count)
+            if celltype != CellType.BITS:
+                regs = exc_simulator.getValues(FX_READ_REG, 30, reg_count)
+            else:
+                reg_bits = exc_simulator.getValues(FX_READ_BIT, 30 * 16, 16)
+                reg_value = sum([ bit * 2 ** i for i, bit in enumerate(reg_bits)])
+                regs = [reg_value]
             if reg_count == 1:
                 assert expect_value == regs[0], f"type({celltype})"
             else:
@@ -561,7 +568,12 @@ class TestSimulator:
         is_int = celltype != CellType.FLOAT32
         reg_count = 1 if celltype in (CellType.BITS, CellType.UINT16) else 2
         for _i in range(100):
-            regs = exc_simulator.getValues(FX_READ_REG, 30, reg_count)
+            if celltype != CellType.BITS:
+                regs = exc_simulator.getValues(FX_READ_REG, 30, reg_count)
+            else:
+                reg_bits = exc_simulator.getValues(FX_READ_BIT, 30 * 16, 16)
+                reg_value = sum([ bit * 2 ** i for i, bit in enumerate(reg_bits)])
+                regs = [reg_value]
             if reg_count == 1:
                 new_value = regs[0]
             else:
@@ -581,9 +593,9 @@ class TestSimulator:
             )
         ),
     )
-    async def test_simulator_server_tcp(self):
+    async def test_simulator_server_tcp(self, unused_tcp_port):
         """Test init simulator server."""
-        task = ModbusSimulatorServer()
+        task = ModbusSimulatorServer(http_port=unused_tcp_port)
         await task.run_forever(only_start=True)
-        await asyncio.sleep(5)
+        await asyncio.sleep(0.5)
         await task.stop()
