@@ -6,8 +6,6 @@ bit based request/response messages:
 * Read/Write Discretes
 * Read Coils
 """
-from test.conftest import MockContext
-
 from pymodbus.file_message import (
     FileRecord,
     ReadFifoQueueRequest,
@@ -18,6 +16,7 @@ from pymodbus.file_message import (
     WriteFileRecordResponse,
 )
 from pymodbus.pdu import ModbusExceptions
+from test.conftest import MockContext  # pylint: disable=wrong-import-order
 
 
 TEST_MESSAGE = b"\x00\n\x00\x08\x00\x01\x00\x02\x00\x03\x00\x04"
@@ -161,10 +160,10 @@ class TestBitMessage:
 
     def test_read_file_record_response_encode(self):
         """Test basic bit message encoding/decoding."""
-        records = [FileRecord(record_data=b"\x00\x01\x02\x03")]
+        records = [FileRecord(record_data=b"\x00\x01\x02\x03\x04\x05")]
         handle = ReadFileRecordResponse(records)
         result = handle.encode()
-        assert result == b"\x06\x06\x02\x00\x01\x02\x03"
+        assert result == b"\x08\x03\x06\x00\x01\x02\x03\x04\x05"
 
     def test_read_file_record_response_decode(self):
         """Test basic bit message encoding/decoding."""
